﻿Imports Infragistics.Windows.DataPresenter
Imports System.Windows.Threading

Class MainWindow

    Private Sub MainWindow_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded
        Dim Employees As New List(Of Employee)
        Employees.Add(New Employee("Tom", "Cruise"))
        Employees.Add(New Employee("Morgan", "Freeman"))
        Employees.Add(New Employee("Brad", "Pit"))
        Employees.Add(New Employee("Tom", "Hanks"))
        xamDataGrid1.DataSource = Employees
    End Sub

    Private Sub MenuItem_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        Dim Menu As MenuItem = CType(e.Source, MenuItem)
        Select Case Menu.Header.ToString.ToLower
            Case "view"
                MessageBox.Show("View")

            Case "edit"
                MessageBox.Show("Edit")

            Case "open..."
                Dim view As New UserControl1
                System.Threading.Thread.Sleep(2000)
                OpenView(view)

        End Select
    End Sub

    Private Sub xamDataGrid1_ContextMenuOpening(ByVal sender As Object, ByVal e As System.Windows.Controls.ContextMenuEventArgs) Handles xamDataGrid1.ContextMenuOpening
        GridContextMenuOpening(e, DirectCast(Resources("GridContextMenu"), ContextMenu))
    End Sub

#Region " Util "

    Private Shared Function GridContextMenuOpening(ByRef e As ContextMenuEventArgs, ByVal Menu As ContextMenu) As Boolean
        Try
            Dim dp As New DataRecordPresenter
            If OpenMenu(e.OriginalSource, dp) AndAlso IsRowClicked(e.OriginalSource) Then
                If dp.IsHeaderRecord Or dp.IsFilterRecord Then
                    Return False
                Else
                    dp.ContextMenu = Menu
                    dp.IsSelected = True
                    dp.IsActive = True
                    Return True
                End If
            Else
                e.Handled = True
                Return False
            End If
        Catch ex As Exception

        End Try
    End Function

    Private Shared Function OpenMenu(ByVal o As Object, ByRef dp As DataRecordPresenter) As Boolean
        Try
            Dim uiElement As DependencyObject = DirectCast(o, DependencyObject)
            While Not (TypeOf uiElement Is DataRecordPresenter)
                uiElement = VisualTreeHelper.GetParent(uiElement)
                If uiElement Is Nothing Then
                    Return False
                End If
                If TypeOf uiElement Is DataRecordPresenter Then
                    dp = uiElement
                    If dp.IsFilterRecord Then Return False
                    If dp.IsHeaderRecord Then Return False
                    If dp.IsEnabled = False Then Return False
                    Return True
                End If
            End While
            Return False
        Catch ex As Exception

        End Try
    End Function

    Private Shared Function IsRowClicked(ByVal o As Object) As Boolean
        Try
            Dim uiElement As DependencyObject = DirectCast(o, DependencyObject)
            While Not (TypeOf uiElement Is DataRecordCellArea)
                uiElement = VisualTreeHelper.GetParent(uiElement)
                If TypeOf uiElement Is DataRecordCellArea Then
                    Return True
                End If
            End While
            Return False
        Catch ex As Exception

        End Try
    End Function

#End Region

#Region " Shell "

    Public Sub OpenView(ByVal view As UserControl)
        Try
            Dim Tab As New TabItem
            Tab.Header = "Details"
            Tab.Content = view
            Tab.Height = 25
            tcShell.Items.Add(Tab)
            ForceFocus(view, Tab)
        Catch ex As Exception

        End Try
    End Sub

#End Region

#Region " Focus "

    Public Shared Sub ForceFocus(ByVal main As UserControl, ByVal element As UIElement)
        main.Dispatcher.BeginInvoke(New Action(Of UIElement)(AddressOf ForceFocusOnElement), DispatcherPriority.Input, element)
    End Sub

    Private Shared Sub ForceFocusOnElement(ByVal element As UIElement)
        element.Focus()
    End Sub

#End Region

End Class

Public Class Employee

    Private _Firstname As String
    Private _Surname As String

    Sub New()
        _Firstname = ""
        _Surname = ""
    End Sub

    Sub New(Firstname As String, Surname As String)
        Me.New()
        _Firstname = Firstname
        _Surname = Surname
    End Sub

    Public Property Firstname As String
        Get
            Return _Firstname
        End Get
        Set(value As String)
            _Firstname = value
        End Set
    End Property

    Public Property Surname As String
        Get
            Return _Surname
        End Get
        Set(value As String)
            _Surname = value
        End Set
    End Property

End Class